package org.bitbucket.bradleysmithllc.etlunit;

import org.bitbucket.bradleysmithllc.etlunit.feature.RuntimeOption;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestClass;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestMethod;

import java.io.File;
import java.io.IOException;
import java.util.List;

public interface RuntimeSupport
{
	String getProjectRelativePath(File fke) throws IOException;
	File getTempDirectory();

	File createTempFile(String name);

	File getGeneratedSourceDirectory(String feature);

	File createGeneratedSourceFile(String feature, String name);

	File getFeatureSourceDirectory(String feature);

	File getSourceDirectory();

	File getTestSourceDirectory();
	File getTestSourceDirectory(String _package);

	/**
	 * Gets the source directory for the currently executing test.
	 */
	File getCurrentTestSourceDirectory();

	File getTestResourceDirectory(String subdir);
	File getTestResourceDirectory(String _package, String subdir);

	/**
	 * Gets the source directory for the currently executing test.
	 */
	File getCurrentTestResourceDirectory(String subdir);

	File getReportDirectory(String type);

	File getFeatureConfigurationDirectory(String feature);

	File getSourceDirectory(String feature);

	File createSourceFile(String feature, String name);

	File getReferenceDirectory(String path_);

	File getReferenceFile(String path_, String name);

	ProcessExecutor getProcessExecutor();

	void installProcessExecutor(ProcessExecutor od);

	ProcessFacade execute(ProcessDescription pd) throws IOException;

	String getProjectName();

	String getProjectVersion();

	String getProjectUser();

	// Returns a unique identifier for this project based on the absolute path to the project and the MAC address of the computer
	String getProjectUID();

	void overrideRuntimeOption(RuntimeOption runtimeOption);
	void overrideRuntimeOptions(List<RuntimeOption> runtimeOptions);

	RuntimeOption getRuntimeOption(String qualifiedName);

	List<RuntimeOption> getRuntimeOptions(String featureName);

	List<RuntimeOption> getRuntimeOptions();

	List<String> getPackages(File root);

	List<String> getReferencePackages(String path);

	List<String> getTestPackages();

	boolean isTestActive();
	String getCurrentlyProcessingTestPackage();
	ETLTestClass getCurrentlyProcessingTestClass();
	ETLTestMethod getCurrentlyProcessingTestMethod();

	File getVendorBinaryDir(String path);

	File getVendorBinaryDir();

	Log getApplicationLog();

	Log getUserLog();

	String getLocalAddress();
}