package org.bitbucket.bradleysmithllc.etlunit.util;

import com.fasterxml.jackson.databind.JsonNode;
import com.github.fge.jackson.JsonLoader;
import com.github.fge.jsonschema.exceptions.ProcessingException;
import com.github.fge.jsonschema.main.JsonSchema;
import com.github.fge.jsonschema.main.JsonSchemaFactory;

public class JsonSchemaLoader
{
	public static JsonSchema fromResource(String classPath)
	{
		return fromResource(classPath, null);
	}

	public static JsonSchema fromResource(String classPath, ClassLoader loader)
	{
		JsonSchemaFactory fact = JsonSchemaFactory.byDefault();

		try {
			ClassLoader classLoader = ObjectUtils.firstNotNull(
					loader,
					Thread.currentThread().getContextClassLoader(),
					ClassLoader.getSystemClassLoader()
			);

			JsonNode jNode = JsonLoader.fromURL(
					classLoader
							.getResource(classPath));

			JsonNode node = JsonSchemaResolver.resolveSchemaReference(jNode);

			JsonSchemaValidator.validateV4Schema(node);

			return fact.getJsonSchema(node);
		} catch (ProcessingException e) {
			throw new RuntimeException("Could not validate schema: " + JsonProcessingUtil.getProcessingReport(e), e);
		} catch (Exception e) {
			throw new RuntimeException("Could not load schema", e);
		}
	}
}